import React from 'react';
import { gameInfo, getGuesses, sendGuess } from "../../services/ggsh.service";
import GuessPane from "./guess-pane.component";
import Loading from '../loading.component';
import PositionedStreetView from "../maps/positioned-street-view.component";

// TODO want a button for moving the pano back to start somehow

const GamePanel = ({
  onSelectPoint,
  onSubmitGuess,
  streetViewPoint,
  selectedPoint,
  roundSeconds,
  onTimeout,
  submitDisabled,
}) => (
  <div style={{
    height: "100%",
    display: "flex",
    justifyContent: "space-between",
    alignItems: "center"
  }}>
    <div style={{ height: "100%", margin: "2px", flex: 3 }}>
        <PositionedStreetView position={streetViewPoint} />
    </div>
    <GuessPane 
      roundSeconds={roundSeconds}
      onTimeout={onTimeout}
      onSelectPoint={onSelectPoint}
      onSubmitGuess={onSubmitGuess}
      submitDisabled={submitDisabled || selectedPoint === null}
    />
  </div>
);

class GamePanelContainer extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      currentRound: null,
      targetPoint: null,
      selectedPoint: null,
      roundTimer: null,
      submitDisabled: false,
    }
  }

  async componentDidMount() {
    const { gameId, playerName } = this.props;
    const { currentRound } = await getGuesses(gameId, playerName);
    if (currentRound) {
      const { coords, timer } = await gameInfo(gameId);
      const targetPoint = coords[currentRound];
      this.setState({
        currentRound,
        targetPoint,
        roundTimer: timer,
      });
    } else {
      const { onGameEnd } = this.props;
      onGameEnd();
    }
  }

  async handleSubmitGuess() {
    this.setState({ submitDisabled: true })
    const { currentRound, selectedPoint, targetPoint } = this.state;
    const { gameId, playerName, onRoundEnd } = this.props;
    const { score, totalScore } = await sendGuess(gameId, playerName, currentRound, selectedPoint || { timeout: true });
    onRoundEnd({
      roundNum: currentRound,
      selectedPoint,
      targetPoint,
      score,
      totalScore,
    });
  }

  render() {
    const { 
      currentRound, 
      targetPoint, 
      selectedPoint, 
      roundTimer, 
      submitDisabled 
    } = this.state;
    if (!currentRound || !targetPoint || !roundTimer) {
      return <Loading/>
    }
  
    return (
      <GamePanel
        onSelectPoint={selectedPoint => this.setState({ selectedPoint })}
        onSubmitGuess={() => this.handleSubmitGuess()}
        streetViewPoint={targetPoint}
        selectedPoint={selectedPoint}        
        roundSeconds={roundTimer}
        onTimeout={() => this.handleSubmitGuess()}
        submitDisabled={submitDisabled}
      />
    );
  }
}

export default GamePanelContainer;
